package com.tappli.android.readmorelistviewtest;

import android.content.Context;
import android.util.AttributeSet;
import android.view.View;
import android.widget.ListAdapter;
import android.widget.ListView;

public class ReadMoreListView extends ListView {
	/** 通常モード */
	public static final int MODE_DEFAULT = 0;
	/** 読み込み中モード */
	public static final int MODE_LOADING = 1;

	// AdapterのgetCount()がMAX以下ならReadMoreを表示する
	private int max = 0;

	// 「もっと読む」が固定表示か、切り替え表示か
	private boolean isFixedType = true;
	// 「もっと読む」を表示するView
	private View readMoreView = null;
	// 通常表示するViewとID
	private View defaultView = null;
	// ローティング時に表示するViewとID
	private View loadingView = null;
	// 現在の表示モード
	private int mode = MODE_DEFAULT;
	// 現在ViewがFooterに登録されているか
	private boolean isAdded = false;

	/**
	 * コンストラクタ
	 * 
	 * @param context
	 * @param attrs
	 */
	public ReadMoreListView(Context context, AttributeSet attrs) {
		super(context, attrs);
	}

	@Override
	protected void handleDataChanged() {
		// Adapterの変化通知を受ける
		super.handleDataChanged();

		updateFooterView();
	}

	/**
	 * 最大値を設定
	 * 
	 * @param max
	 */
	public void setMax(int max) {
		this.max = max;
		// 表示内容を更新
		updateFooterView();
	}

	/**
	 * 「もっと読む」が表示されるかどうか
	 * 
	 * @return
	 */
	public boolean isVisibleReadMoreView() {
		ListAdapter adapter = getAdapter();
		if (adapter == null) {
			return false;
		}

		// リストの件数を比較する
		int count = adapter.getCount();

		// 0件では「もっと読む」は表示しない -> false
		// 最大件数に達しているので「もっと読む」は表示しない -> false
		return !(count <= 0 || count >= max);
	}
	
	/**
	 * 「もっと読む」の表示状態を取得
	 * @return
	 */
	public int getMode() {
		// 固定表示モードなら常に通常モード
		if (isFixedType) {
			return MODE_DEFAULT;
		}
		
		return mode;
	}

	/**
	 * 「もっと読む」の表示内容を切り替える
	 * 
	 * @param mode
	 */
	public void setMode(int mode) {
		// 固定表示モードなら何もしない
		if (isFixedType) {
			return;
		}

		// Viewが未設定なら何もしない
		if (readMoreView == null || defaultView == null || loadingView == null) {
			return;
		}

		// 表示・非表示の切り替え
		switch (mode) {
		case MODE_DEFAULT:
			defaultView.setVisibility(VISIBLE);
			loadingView.setVisibility(GONE);
			break;
		case MODE_LOADING:
			defaultView.setVisibility(GONE);
			loadingView.setVisibility(VISIBLE);
			break;
		default:
			return;
		}

		this.mode = mode;
	}

	/**
	 * 固定表示タイプの「もっと読む」Viewを設定する
	 * 
	 * @param view
	 */
	public void setReadMoreView(View view) {
		// 既存のViewがあれば削除
		if (readMoreView != null) {
			removeFooterView(readMoreView);
		}

		this.isFixedType = true;
		this.readMoreView = view;

		// 「もっと読む」を追加
		resetFooterView();
	}

	/**
	 * 切り替えタイプの「もっと読む」Viewを設定する
	 * 
	 * @param view
	 * @param defaultViewId
	 *            VIew内の通常表示するViewのID
	 * @param loadingViewId
	 *            View内のロード中に表示するViewのID
	 */
	public void setReadMoreView(View view, int defaultViewId, int loadingViewId) {
		// 既存のViewがあれば削除
		if (readMoreView != null) {
			removeFooterView(readMoreView);
		}

		this.isFixedType = false;
		this.readMoreView = view;

		// 各View取得
		defaultView = readMoreView.findViewById(defaultViewId);
		loadingView = readMoreView.findViewById(loadingViewId);

		// 「もっと読む」を追加
		resetFooterView();

		// 表示モード設定
		setMode(mode);
	}

	/**
	 * 「もっと読む」を追加する(Viewを替えるときに使用する)
	 */
	private void resetFooterView() {
		if (isVisibleReadMoreView()) {
			// アダプタを再設定する必要があるので取得しておく
			ListAdapter adapter = getAdapter();

			// 「もっと読む」を表示する
			addFooterView(readMoreView);
			isAdded = true;

			// アダプタを設定しなおす
			if (adapter != null) {
				setAdapter(adapter);
			}
		}
	}

	/**
	 * 現在の状況を判断して「もっと読む」を設定する(しない)
	 */
	private void updateFooterView() {
		if (isVisibleReadMoreView()) {
			// すでに登録されていたら何もしない
			if (isAdded) {
				return;
			}

			// Viewを更新
			resetFooterView();
		} else {
			// 表示されていなければ何もしない
			if (!isAdded) {
				return;
			}

			// Viewを剥がす
			removeFooterView(readMoreView);
			isAdded = false;
		}
	}

}
